
#Returns the Next Image Number in directory for Saving
def findNextImage():
    import os
    #Finding all of the Previous Images
    previousPhotos = [i for i in os.listdir('.') if '.jpg' in i]
    if previousPhotos:
        nextNum = max([int(i.split('_')[-1].split('.')[0]) for i in previousPhotos])+1
    else:
        nextNum = 1
    return nextNum

#Connects to the Camera
def connectCamera(cameraIndex = 0):
    import cv2
    #Initialize the camera
    cam = cv2.VideoCapture(cameraIndex)   # 0 -> index of camera
    return cam

#Takes an Image and Returns the Image Array
def takeImage(camera):
    import cv2
    #Load image
    s, img = camera.read()
    #Turn the Image to Greyscale
    grey = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    return grey

#Returns the Angle from two Positions.
#Input (xCenter,yCenter),(xDot,yDot)
#Output currentAngle
def getAngle(center,dot):
    import math
    #Correcting for the offsets.
    x = dot[0] - center[0]
    y = dot[1] - center[1]
    #Calculating the Angle from xy
    angle = math.atan2(y,x)
    degrees = math.degrees(angle)
    #Dealing with the Negative Degrees
    if degrees < 0:
        degrees = 360 + degrees
    return degrees


def processImage(image, showImage = False, saveImage = False):
    import cv2
    import numpy as np
    import time

    #Displaying our Detection
    if showImage:
        cv2.namedWindow("cam-test",cv2.WINDOW_AUTOSIZE)
        cv2.imshow("cam-test",image)
        cv2.waitKey(0)
        cv2.destroyWindow("cam-test")

    # binary thresholding
    #_, thresh = cv2.threshold(grey, 150, 255, cv2.THRESH_BINARY)
    thresh = cv2.adaptiveThreshold(image,255,cv2.ADAPTIVE_THRESH_GAUSSIAN_C,cv2.THRESH_BINARY,11,3)

    #Displaying our Detection
    if showImage:
        cv2.namedWindow("cam-test",cv2.WINDOW_AUTOSIZE)
        cv2.imshow("cam-test",thresh)
        cv2.waitKey(0)
        cv2.destroyWindow("cam-test")

    # detect the big circle
    circles = cv2.HoughCircles(thresh, cv2.HOUGH_GRADIENT, 1, 1000, param1=50, param2=10, minRadius=100, maxRadius=200)

    # make sure only one big circle is detected
    assert len(circles[0]) == 1
    circle = circles[0][0]

    # big circle parameters
    x1, y1, radius1 = circle
    #print(f'disc: x = {x1}, y = {y1}, radius = {radius1}')

    # crop out the background
    cropped_thresh = thresh[int(y1 - radius1):int(y1 + radius1), int(x1 - radius1):int(x1 + radius1)]

    #Displaying our Detection
    if showImage:
        cv2.namedWindow("cam-test",cv2.WINDOW_AUTOSIZE)
        cv2.imshow("cam-test",cropped_thresh)
        cv2.waitKey(0)
        cv2.destroyWindow("cam-test")

    # detect the small circle
    circles = cv2.HoughCircles(cropped_thresh, cv2.HOUGH_GRADIENT, 1, 1000, param1=50, param2=10, minRadius=2, maxRadius=8)

    # make sure only one small circle is detected
    assert len(circles[0]) == 1
    circle = circles[0][0]

    # small circle parameters
    x2, y2, radius2 = circle
    x2 += (x1 - radius1)
    y2 += (y1 - radius1)
    #print(f'dot: x = {x2}, y = {y2}, radius = {radius2}')

    # draw circles
    cv2.circle(image, (x1, y1), int(radius1), (0, 255, 0), 2)
    cv2.circle(image, (x2, y2), int(radius2), (0, 255, 0), 2)

    #Displaying our Detection
    if showImage:
        cv2.namedWindow("cam-test",cv2.WINDOW_AUTOSIZE)
        cv2.imshow("cam-test",image)
        cv2.waitKey(0)
        cv2.destroyWindow("cam-test")

    #Saving our Image
    if saveImage:
        cv2.imwrite('detected_' + str(findNextImage()) + '.jpg', image)

    return getAngle((x1,y1),(x2,y2))

#Processes a list of Images.
def processLOI(listOfImages):
    angles = []
    for image in listOfImages:
        try:
            angles.append(processImage(image))
        except:
            error = 1
    return angles
        

def logError(replicates = 100,saveFile = 'replicates.csv'):
    import csv
    with open(saveFile, 'w') as outFile:
        csvWriter = csv.writer(outFile)
        for i in range(0,replicates):
            csvWriter.writerow([gatherData()])

#Displays the Current Camera View
def calibrationImage(image):
    import cv2
    #Finding our Center.
    center = (int(image.shape[1]/2),int(image.shape[0]/2))
    #Drawing our Circles
    image = cv2.circle(image, center, 150, (255,0,0), 2)
    image = cv2.circle(image, center, 300, (255,0,0), 2)
    #Displaying our Detection
    cv2.namedWindow("cam-test",cv2.WINDOW_AUTOSIZE)
    cv2.imshow("cam-test",image)
    cv2.waitKey(0)
    cv2.destroyWindow("cam-test")


#Calculates the Difference in a List of Positions.
def calculateAngle(lop):
    percentageClose = 0.10
    #Averaging the Start Points
    startPoints = [i for i in lop if i > (1 - percentageClose) * lop[0] and i < (percentageClose + 1) * lop[0] ]
    startPoint = sum(startPoints)/len(startPoints)
    #Averaging the End Points
    endPoints = [i for i in lop if i > (1 - percentageClose) * lop[-1] and i < (percentageClose + 1) * lop[-1] ]
    endPoint = sum(endPoints)/len(endPoints)
    #Logic for the Amount of Degrees changed.
    if endPoint < startPoint:
        #If End is Less Degrees than Start, we've passed 360.
        delta = (360-startPoint) + endPoint
    else:
        delta = endPoint - startPoint
    return delta

if __name__ == "__main__":
    #logError()
    camera = connectCamera(0)
    #calibrationImage(takeImage(camera))
    print(processImage(takeImage(camera),showImage=True))
