import detection
import motorControl
import datetime
import time

class testObject():
    def __init__(self,motorPort='COM6',cameraNumber=0):
        self._connect(motorPort,cameraNumber)
    #Connects to the Motor and Camera.
    def _connect(self,motorPort,cameraNumber):
        self.serialConnection = motorControl.connect(port = motorPort)
        self.cameraConnection = detection.connectCamera(cameraIndex = cameraNumber)
    #Closes connections to the Motor and Camera.
    def disconnect(self):
        self.serialConnection.close()
        self.cameraConnection.release()
    #Runs a Test Moving a Set Distance, Returns the Result.
    def runTest(self,motorTime,motorPwm,stopTime,stopPwm):
        #Getting to our Default Position first
        self.jogMotor()
        #Taking some Initial Photos
        positions = [detection.takeImage(self.cameraConnection) for i in range(0,10)]
        #Moving the Wheel
        self._moveMotor(motorTime,motorPwm,stopTime,stopPwm)
        #Grabbing some Extra Images after the Stop
        positions = positions + [detection.takeImage(self.cameraConnection) for i in range(0,10)]
        #Processing the Data
        #Getting rid of the ones that fail image reconition
        positions = detection.processLOI(positions)
        #Calculating the Angle from Start to End.
        angle = detection.calculateAngle(positions)
        return angle
    #Moves the Motor
    def _moveMotor(self,motorTime,motorPwm,stopTime,stopPwm):
        #Start time before we send the Command
        startTime = datetime.datetime.now()
        motorControl.sendCommand(self.serialConnection, motorTime,motorPwm,stopTime,stopPwm)
        #Determining how long we should wait
        waitTime = (motorTime + stopTime)/1000
        while (datetime.datetime.now() - startTime).seconds < waitTime:
            time.sleep(0.1)
        return
    #Jogs Until the Motor is in the Right Position
    #endPosition = Degrees we aim to be at with the location
    #maxError = How far off we can be from this location
    #jogTime = How much impulse we give the motor to get there.
    def jogMotor(self,endPosition = 0, maxError = 5):
        i = 0
        #Getting our Initial Position
        position = self._getPosition()
        delta = abs(position-endPosition)
        #Rotating wheel until we're close to where we want to be.
        while delta > maxError:
            #Using the Distance to Control Intensity
            if delta > 80:
                jogTime = 10
            elif delta > 40:
                jogTime = 5
            else:
                jogTime = 3
            #Moving the Motor, Calculating new Delta
            self._moveMotor(jogTime,255,1,100)
            position = self._getPosition()
            delta = abs(position-endPosition)
            i = i + 1
            if i > 50:
                print('TenX Fail')
                break
        #Returning where we ended.
        return(position)
    #Returns a Position from the Camera.
    def _getPosition(self,images=5):
        angles = []
        i = 0
        while not angles:
            angles = detection.processLOI([detection.takeImage(self.cameraConnection) for i in range(0,images)])
            i = i + 1
            #If we Fail 3x, Just return and keep going.
            if i > 3:
                return 999.0
        return sum(angles)/len(angles) * 1.0




def runTest(motorTime,motorPwm,stopTime,stopPwm):
    #Connecting to our Motor
    serialConnection = motorControl.connect()
    cameraConnection = detection.connectCamera()
    #Taking some Initial Photos
    positions = [detection.takeImage(cameraConnection) for i in range(0,10)]
    #Start time before we send the Command
    startTime = datetime.datetime.now()
    motorControl.sendCommand(serialConnection, motorTime,motorPwm,stopTime,stopPwm)
    #Determining how long we should wait
    waitTime = (motorTime + stopTime)/1000
    while (datetime.datetime.now() - startTime).seconds < waitTime:
        #time.sleep(0.01)
        #Getting Images while Moving
        positions.append(detection.takeImage(cameraConnection))
    #Grabbing some Extra Images after the Stop
    positions = positions + [detection.takeImage(cameraConnection) for i in range(0,10)]
    #Processing the Data
    positions = detection.processLOI(positions)
    serialConnection.close()
    return positions

def plotPositions(positions):

    import matplotlib.pyplot as plt

    fig, ax = plt.subplots()
    ax.plot([i for i in range(0,len(positions))], positions)

    ax.set(xlabel='Datapoint (#)', ylabel='Location (Degrees)',
        title='Movement Data')
    ax.grid()

    #fig.savefig("test.png")
    plt.show()

if __name__ == "__main__":
    #positions = runTest(35,255,1,100)
    #print(detection.calculateAngle(positions))
    """
    a = testObject()
    i = 0
    while i < 20:
        print(a.runTest(35,255,1,100))
        i += 1
    a.disconnect()
    """
    #Running Single Test
    a = testObject()
    print(a.runTest(325,40,1,100))
    a.disconnect()
    pass